ParserIterator

Struct ParserIterator 

pub struct ParserIterator<T> { /* private fields */ }
Iterator returned from BlockParser::parse that allows for advanced transformations.

Implementations

Source

impl<A: Send + 'static> ParserIterator<A>

pub fn with_height(&self) -> ParserIterator<(usize, A)>

Adds the block height to this iterator.

pub fn ordered(&self) -> ParserIterator<A>

Orders the results by block height, can be called for a small increase in memory and runtime.

Example

Using the ordered function to get the first 10 block hashes in-height order:

use bitcoin_block_parser::blocks::*;
use bitcoin::BlockHash;

let parser = BlockParser::new("/home/user/.bitcoin/blocks/").unwrap();
let ordered: ParserIterator<BlockHash> = parser.parse(|block| block.block_hash());
let first_10: Vec<BlockHash> = ordered.take(10).collect();
pub fn map_parallel<B: Send + 'static>( &self, function: impl Fn(A) -> B + Clone + Send + 'static, ) -> ParserIterator<B>

Perform a map function using multiple threads.

pub fn pipeline_fn<B: Send + 'static, C: Send + 'static>( &self, f1: impl Fn(A) -> B + Clone + Send + 'static, f2: impl Fn(B) -> C + Clone + Send + 'static, ) -> ParserIterator<C>

Pipelines allow you to perform two functions on the same batch of blocks. Useful when you want multithreaded performance while processing blocks in-order.

Example

For example if calculating the size difference between consecutive blocks you could use the following code:

use bitcoin_block_parser::blocks::*;
use std::collections::HashMap;
use std::convert::identity;
use bitcoin::BlockHash;
use bitcoin::hashes::Hash;

let parser = BlockParser::new("/home/user/.bitcoin/blocks/").unwrap();
let mut block_sizes: HashMap<BlockHash, isize> = HashMap::new();
let mut differences = vec![];
// Initial block size for the genesis block
block_sizes.insert(BlockHash::all_zeros(), 0);

for block in parser.parse(identity).ordered() {
    // Store this block's size in the shared state
    let block_size = block.total_size() as isize;
    block_sizes.insert(block.block_hash(), block_size);
    // Look up the previous size to compute the difference
    let prev_block_hash = block.header.prev_blockhash;
    let prev_size = block_sizes.remove(&prev_block_hash);
    differences.push(block_size - prev_size.unwrap());
}

let max_difference = differences.into_iter().max().unwrap();
println!("Maximum increase in block size: {}", max_difference);

The previous code runs on a single thread. If we want to leverage multithreading we can use pipeline functions for a large speed-up:

use bitcoin_block_parser::blocks::*;
use dashmap::DashMap;
use std::convert::identity;
use std::sync::Arc;
use bitcoin::BlockHash;
use bitcoin::hashes::Hash;

let parser = BlockParser::new("/home/user/.bitcoin/blocks/").unwrap();
// State shared across all threads
let block_sizes: Arc<DashMap<BlockHash, isize>> = Arc::new(DashMap::new());
let blocksizes_clone = block_sizes.clone();
// Initial block size for the genesis block
block_sizes.insert(BlockHash::all_zeros(), 0);

let iterator = parser.parse(identity).ordered().pipeline_fn(
    move |block| {
        // Store this block's size in the shared state
        let block_size = block.total_size() as isize;
        block_sizes.insert(block.block_hash(), block_size);
        (block.header.prev_blockhash, block_size)
    },
    move |(prev_block_hash, block_size)| {
        // Look up the previous size to compute the difference
        let prev_size = blocksizes_clone.remove(&prev_block_hash);
        block_size - prev_size.unwrap().1
    },
);

let max_difference = iterator.max().unwrap();
println!("Maximum increase in block size: {}", max_difference);
pub fn pipeline<B: Send + 'static, C: Send + 'static>( &self, pipeline: &(impl Pipeline<A, B, C> + Clone + Send + 'static), ) -> ParserIterator<C>

Runs ParserIterator::pipeline_fn functions by implementing a Pipeline trait for convenience / cleaner code.